==> tensorstore-0.1.44.tar.gz: tensorstore-0.1.44/tools/cmake/bazel_to_cmake/starlark/struct.py <==
# Copyright 2022 The TensorStore Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""starlark struct function.

https://bazel.build/rules/lib/struct
"""

from typing import Any

# pylint: disable=missing-function-docstring


class Struct(object):

  def __init__(self, **kwargs):
    self.__dict__['_hidden_fields'] = sorted(kwargs.keys())
    self.__dict__.update(kwargs)

  @property
  def _fields(self):
    return self.__dict__['_hidden_fields']

  def __repr__(self):
    kwargs_repr = ','.join(
        f'{k}={repr(self.__dict__.get(k))}' for k in self._fields
    )
    return f'struct({kwargs_repr})'

  def __add__(self, addend: 'Struct'):
    """In Starlark, struct + struct = struct."""
    common_fields = set(self._fields) & set(addend._fields)
    if common_fields:
      raise ValueError(
          'Cannot concat structs with common field(s): {}'.format(
              ', '.join(common_fields)
          )
      )

    fields = {k: self.__dict__.get(k) for k in self._fields}
    fields.update({k: addend.__dict__.get(k) for k in addend._fields})
    return self.__class__(**fields)

  def __eq__(self, other):
    if not isinstance(other, Struct):
      return False
    if self._fields != other._fields:
      return False
    for x in self._fields:
      if self.__dict__.get(x) != other.__dict__.get(x):
        return False
    return True

  def __ne__(self, other):
    if not isinstance(other, Struct):
      return True
    if self._fields != other._fields:
      return True
    for x in self._fields:
      if self.__dict__.get(x) != other.__dict__.get(x):
        return True
    return False

  def __setattr__(self, *ignored):
    raise NotImplementedError

  def __delattr__(self, *ignored):
    raise NotImplementedError

  def __getattr__(self, attr: str) -> Any:
    return self.__dict__[attr]

==> tensorstore-0.1.75.tar.gz: tensorstore-0.1.75/tools/cmake/bazel_to_cmake/starlark/struct.py <==
# Copyright 2022 The TensorStore Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""starlark struct function.

https://bazel.build/rules/lib/struct
"""

from typing import Any

# pylint: disable=missing-function-docstring


class Struct(object):

  def __init__(self, **kwargs):
    self.__dict__['_hidden_fields'] = sorted(kwargs.keys())
    self.__dict__.update(kwargs)

  @property
  def _fields(self):
    return self.__dict__['_hidden_fields']

  def __repr__(self):
    kwargs_repr = ','.join(
        f'{k}={repr(self.__dict__.get(k))}' for k in self._fields
    )
    fn = self.__class__.__name__
    if fn == 'Struct' or fn == 'GenericProvider':
      fn = 'struct'
    return f'{fn}({kwargs_repr})'

  def __add__(self, addend: 'Struct'):
    """In Starlark, struct + struct = struct."""
    common_fields = set(self._fields) & set(addend._fields)
    if common_fields:
      raise ValueError(
          'Cannot concat structs with common field(s): {}'.format(
              ', '.join(common_fields)
          )
      )

    fields = {k: self.__dict__.get(k) for k in self._fields}
    fields.update({k: addend.__dict__.get(k) for k in addend._fields})
    return self.__class__(**fields)

  def __eq__(self, other):
    if not isinstance(other, Struct):
      return False
    if self._fields != other._fields:
      return False
    for x in self._fields:
      if self.__dict__.get(x) != other.__dict__.get(x):
        return False
    return True

  def __ne__(self, other):
    return not self.__eq__(other)

  def __setattr__(self, *ignored):
    raise NotImplementedError

  def __delattr__(self, *ignored):
    raise NotImplementedError

  def __getattr__(self, attr: str) -> Any:
    return self.__dict__[attr]
